/*
   Copyright 2018 Brigham Young University

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
const date2Str = dt => `${dt.getUTCMonth() + 1}/${dt.getUTCDate()}/${dt.getUTCFullYear()}`

export const dateTimeFormat = value => {
  if (!value) return value
  const fmt = dt => dt.toLocaleDateString() + ' | ' + dt.toLocaleTimeString()
  if (value.toLocaleDateString && value.toLocaleTimeString) {
    return fmt(value)
  }
  return fmt(new Date(value))
}

export const dateFormat = value => {
  if (!value) return value
  const fmt = date2Str
  if (value.toUTCString) {
    return fmt(value)
  }
  return fmt(new Date(value))
}

export const timeFormat = value => {
  if (!value) return value
  const fmt = dt => dt.toLocaleTimeString()
  if (value.toLocaleTimeString) {
    return fmt(value)
  }
  return fmt(new Date(value))
}

// V2 versions of these functions were written to allow Ensign to view applications based on their new block schedule.
export const yearTermFormat = value => {
  if (!value) return value
  if (!/^[0-9]{4}[1345]$/.test(value)) return value
  const term = {
    1: 'W',
    3: 'Sp',
    4: 'Su',
    5: 'F'
  }[value.substr(-1)]
  return value.substr(0, 4).concat(term)
}

export const yearTermFormatV2 = value => {
  if (!value) return value
  if (!/^[0-9]{4}[1-6]$/.test(value)) return value
  const term = {
    1: 'W1',
    2: 'W2',
    3: 'S1',
    4: 'S2',
    5: 'F1',
    6: 'F2'
  }[value.substr(-1)]
  return value.substr(0, 4).concat(term)
}

export const yearTermLongFormat = value => {
  if (!value) return value
  if (!/^[0-9]{4}[1345]$/.test(value)) return value
  const term = {
    1: 'Winter',
    3: 'Spring',
    4: 'Summer',
    5: 'Fall'
  }[value.substr(-1)]
  return `${term} ${value.substr(0, 4)}`
}

export const yearTermLongFormatV2 = value => {
  if (!value) return value
  if (!/^[0-9]{4}[1-6]$/.test(value)) return value
  const term = {
    1: 'Winter 1',
    2: 'Winter 2',
    3: 'Spring 1',
    4: 'Spring 2',
    5: 'Fall 1',
    6: 'Fall 2'
  }[value.substr(-1)]
  return `${term} ${value.substr(0, 4)}`
}
