const utConversionTable = {
  'A+': 4.0,
  'A': 4.0,
  'A-': 3.7,
  'B+': 3.4,
  'B': 3.0,
  'B-': 2.7,
  'C+': 2.4,
  'C': 2.0,
  'C-': 1.7,
  'D+': 1.4,
  'D': 1.0,
  'D-': 0.7,
  'E': 0.0,
  'F': 0.0,
  'P': 0.0,
  'NG': 0.0,
  'NC': 0.0,
  'I': 0.0,
  'W': 0.0,
  'WP': 0.0
}

const letterCoversionTable = {
  'A/A+': 4.0,
  'A+': 4.0,
  'A': 4.0,
  'A-': 3.7,
  'B+': 3.3,
  'B': 3.0,
  'B-': 2.7,
  'C+': 2.3,
  'C': 2.0,
  'C-': 1.7,
  'D+': 1.3,
  'D': 1.0,
  'D-': 0.7,
  'E': 0,
  'F': 0,
  'Pass': 0,
  'Fail': 0,
  'No Grade': 0,
  'No Credit': 0,
  'In Progress': 0,
  'Incomplete': 0,
  'Withdrew': 0
}

const percentageConversionTable = {
  '100+': 4.0,
  '100': 4.0,
  '99': 4.0,
  '98': 4.0,
  '97': 4.0,
  '96': 4.0,
  '95': 4.0,
  '94': 4.0,
  '93': 4.0,
  '92': 3.7,
  '91': 3.7,
  '90': 3.7,
  '89': 3.3,
  '88': 3.3,
  '87': 3.3,
  '86': 3.0,
  '85': 3.0,
  '84': 3.0,
  '83': 3.0,
  '82': 2.7,
  '81': 2.7,
  '80': 2.7,
  '79': 2.3,
  '78': 2.3,
  '77': 2.3,
  '76': 2.0,
  '75': 2.0,
  '74': 2.0,
  '73': 2.0,
  '72': 1.7,
  '71': 1.7,
  '70': 1.7,
  '69': 1.3,
  '68': 1.3,
  '67': 1.3,
  '66': 1.0,
  '65': 1.0,
  '<65': 0,
  'Pass': 0,
  'Fail': 0,
  'No Grade': 0,
  'No Credit': 0,
  'In Progress': 0,
  'Incomplete': 0,
  'Withdrew': 0
}

/*
 * const fixedToFloat = (fixed) => {
 *   return fixed / 100
 * }
 *
 * const fixedToString = (fixed) => {
 *   return fixedToFloat(fixed).toFixed(2)
 * }
 */

const convertGrade = (state) => (value) => {
  // if (state === 'UT') return utConversionTable[value] || 0 TODO handle UT service grades?
  const asLetter = letterCoversionTable[value]
  if (asLetter !== undefined) return asLetter
  return percentageConversionTable[value] || 0
}

const convCredit = credit => {
  switch (credit) {
    case 'YEAR':
      return 1.0 // return 12
    case 'SEM':
      return 0.5 // return 6
    case 'TRI':
      return 0.333 // return 4
    case 'QRTR':
      return 0.25 // return 3
  }
  return 0
}

export function calculateGpa (classes = [], state) {
  const gradeConverter = convertGrade(state)
  const converted = classes.map(c => {
    const ignoredGrades = [
      'Pass',
      'Fail',
      'No Grade',
      'No Credit',
      'In Progress',
      'Incomplete',
      'Withdrew',
      'P',
      'NG',
      'NC',
      'I',
      'W',
      '  WP',
      '',
      null
    ]
    const validGrades = c.grades.filter(g => !ignoredGrades.includes(g))

    return {
      terms: validGrades.length,
      credits: convCredit(c.credits),
      points: validGrades.map(gradeConverter)
    }
  })
  const totalCredits = converted.reduce((t, c) => t + c.credits * c.terms, 0)
  const gpa = converted.reduce((t, c) => {
    const cred = c.credits
    return t + c.points.reduce((t, c) => t + c * cred, 0)
  }, 0) / totalCredits
  return gpa.toFixed(2)
}
