import get from 'lodash.get'
import ageFn from 's-age'

export const parseBasic = b => {
  const admitPeriod = get(b, 'admit_period.value')
  const applicantId = get(b, 'applicant_id.value')
  const applicantMajor = get(b, 'admit_major.description')
  const applicantType = get(b, 'applicant_type.value')
  const firstname = get(b, 'first_name.value')
  const surname = get(b, 'surname.value')
  const applicantName = `${firstname} ${surname}`
  const sortName = `${surname}, ${firstname}`
  const preferredName = get(b, 'preferred_name.value')
  const previousName = get(b, 'previous_name.value')

  const emailAddress = get(b, 'email_address.value')
  const phoneNumber = get(b, 'phone_number.value')
  const countryCode = get(b, 'country_code.value')
  const countryName = countryCode === 'USA' ? '' : get(b, 'country_code.description')
  const addressLines = [
    get(b, 'address_line_1.value', ''),
    get(b, 'address_line_2.value', ''),
    get(b, 'address_line_3.value', ''),
    get(b, 'address_line_4.value', ''),
    get(b, 'address_line_5.value', '')
  ].filter(v => !!(v.trim()))
  if (!addressLines.find(l => l.includes(countryName))) {
    addressLines.push(countryName)
  }
  const city = get(b, 'city.value')
  const state = get(b, 'state_code.description')
  const stateCode = get(b, 'state_code.value')
  const zipCode = get(b, 'postal_code.value')
  if (countryCode === 'USA' && !addressLines.find(l => l.includes(city))) {
    addressLines.push(`${city}, ${stateCode} ${zipCode}`)
  }

  const schoolPreferenceOrder = get(b, 'school_preference_order.value')

  const personId = get(b, 'institution_id.value')
  const netId = get(b, 'net_id.value')

  const applicationDecision = get(b, 'decision.description')
  const applicationStatus = get(b, 'application_status.description')
  const submittedOn = get(b, 'submitted_date_time.value')
  return {
    addressLines,
    admitPeriod,
    applicantId,
    applicantMajor,
    applicantName,
    applicantType,
    applicationStatus: get(b, 'decision.value', 'O') === 'O' ? applicationStatus : applicationDecision,
    city,
    emailAddress,
    netId,
    personId,
    phoneNumber,
    preferredName,
    previousName,
    schoolPreferenceOrder,
    sortName,
    state,
    submittedOn,
    surname,
    zipCode
  }
}

export const parseBioClassifications = b => {
  const parseLanguages = ({ value }) => (value)
  const parseRacialCategories = ({ value }) => (value)
  const biographical = {
    languages: get(b, 'languages', []).map(parseLanguages),
    racialCategories: get(b, 'racial_categories', []).map(parseRacialCategories),
    residingWith: get(b, 'residing_with.value'),
    singleParent: get(b, 'single_parent.value'),
    fathersLevelOfEducation: get(b, 'fathers_level_of_education.value'),
    mothersLevelOfEducation: get(b, 'mothers_level_of_education.value'),
    hispanicFlag: get(b, 'hispanic_flag.value'),
    familyIncome: get(b, 'family_income.value'),
    relatedAlumni: get(b, 'alumni.value')
  }
  return biographical
}

export const parseAppBackgroundQuestions = q => {
  return {
    convictionRecord: get(q, 'conviction_record.value'),
    criminalConvictions: get(q, 'criminal_convictions', []).map(c => {
      return {
        crime: get(c, 'crime.value'),
        crimeYear: get(c, 'crime_year.value'),
        courts: get(c, 'courts.value'),
        conditions: get(c, 'conditions.value'),
        probation: get(c, 'probation.value'),
        probationExplanation: get(c, 'probation_explanation.value'),
        penalties: get(c, 'penalties.value'),
        penaltiesExplanation: get(c, 'penalties_explanation.value'),
        additionalExplanation: get(c, 'additional_explanation.value')
      }
    }),
    universityDiscipline: get(q, 'university_discipline.value'),
    disciplineExplanation: get(q, 'university_discipline_explanation.value'),
    pluralMarriage: get(q, 'plural_marriage.value'),
    pluralMarriageExplanation: get(q, 'plural_marriage_explanation.value'),
    sexRegistry: get(q, 'sex_registry.value'),
    sexRegistryExplanation: get(q, 'sex_registry_explanation.value')
  }
}

export const parseMissions = (mission, personal) => {
  const convertType = type => {
    switch (type) {
      case 'F': return 'Full-Time'
      case 'P': return 'Part-Time'
      case 'S': return 'Service'
      default: return ''
    }
  }
  return {
    name: get(mission, 'mission_name.value'),
    service: get(mission, 'mission_service.value'),
    startMonth: get(mission, 'start_month.value'),
    endMonth: get(mission, 'end_month.value'),
    language: get(mission, 'mission_language.value'),
    type: get(mission, 'mission_type.value'),
    typeDisplay: convertType(get(mission, 'mission_type.value')),
    unitNumber: get(mission, 'unit_number.value'),
    unitName: get(mission, 'unit_name.value'),
    missionEmail: get(mission, 'mission_email.value'),
    religionTenure: get(personal, 'religion_tenure.value'),
    religionCode: get(personal, 'religion_code.value'),
    religion: get(personal, 'religion_code.description')
  }
}

export const parseCollegeCumulative = a => {
  return {
    cumulativeGpa: get(a, 'cumulative_gpa.value', ''),
    cumulativeCredits: get(a, 'cumulative_credit.value', '')
  }
}

export const parseCollegeSummaries = a => {
  const collegeSummaries = get(a, 'values', []).map(c => {
    return {
      college: get(c, 'college.description'),
      collegeGpa: get(c, 'gpa_verified.value') || get(c, 'gpa.value'),
      creditHoursGraded: get(c, 'credit_hours_graded.value'),
      dateTimeVerified: get(c, 'date_time_verified.value'),
      degree: get(c, 'degree.value'),
      degreeDate: get(c, 'degree_date.value'),
      endMonth: get(c, 'end_month.value'),
      gradePointsEarned: get(c, 'grade_points_earned.value'),
      major: get(c, 'degree_major.value'),
      startMonth: get(c, 'start_month.value')
    }
  })
  return collegeSummaries
}

export const parseHighSchoolSummaries = a => {
  const parseHsClass = c => {
    return {
      ap: get(c, 'ap_ib_flag.value') === 'Y',
      courseName: get(c, 'course_name.value'),
      credits: get(c, 'credits.value'),
      gradeLevel: get(c, 'grade_level.value'),
      grades: get(c, 'grades').map(g => g.value),
      gradingTerms: get(c, 'grading_terms.value'),
      repeated: get(c, 'repeated.value') === 'Y',
      subject: get(c, 'subject.value')
    }
  }
  const sortHsClasses = (a, b) => {
    return a.gradeLevel - b.gradeLevel
  }
  const highSchoolSummaries = get(a, 'values', []).map(h => {
    return {
      schoolCode: get(h, 'high_school.value'),
      highSchool: get(h, 'high_school.description'),
      highSchoolType: get(h, 'high_school_type.value'),
      startMonth: get(h, 'start_month.value'),
      endMonth: get(h, 'end_month.value'),
      graduationStatus: get(h, 'graduation_status.value'),
      graduationDate: get(h, 'graduation_date.value'),
      highSchoolGpa: get(h, 'high_school_gpa.value'),
      officialGpa: get(h, 'official_gpa.value'),
      gpaCalculated: get(h, 'gpa_calculated.value'),
      concurrentEnrollment: get(h, 'concurrent_enrollment.value'),
      updatedDateTime: get(h, 'updated_date_time.value'),
      updatedById: get(h, 'updated_by_id.value'),
      dateTimeVerified: get(h, 'date_time_verified.value'),
      highSchoolCountry: get(h, 'high_school_country.value'),
      highSchoolState: get(h, 'high_school_state.value'),
      highSchoolCity: get(h, 'high_school_city.value'),
      linkToDocumentImage: get(h, 'link_to_document_image.value'),
      linkToSecondaryDocumentImage: get(h, 'link_to_secondary_document_image.value'),
      transcriptSource: get(h, 'transcript_source.value'),
      apIbFullClasses: get(h, 'ap_ib_full_classes.value'),
      apIbHalfClasses: get(h, 'ap_ib_half_classes.value'),
      listOfClasses: get(h, 'list_of_classes').map(parseHsClass).sort(sortHsClasses)
    }
  })
  return highSchoolSummaries
}

export const parseTestScores = a => {
  const parseTestComponents = list => {
    return {
      name: get(list, 'component_name.value'),
      score: get(list, 'component_score.value')
    }
  }
  const testScores = get(a, 'values', []).map(t => {
    return {
      testDate: get(t, 'test_date.value'),
      dateProcessed: get(t, 'date_time_reported.value'),
      testName: get(t, 'test_name.value'),
      testComponents: get(t, 'test_components').map(parseTestComponents),
      compositeScore: get(t, 'composite_score.value'),
      dateTimeReported: get(t, 'date_time_reported.value'),
      selfReported: get(t, 'self_reported.value', false) === 'Y',
      optIn: get(t, 'test_opt_in.value', 'Y') === 'Y',
      verified: /^[YV]$/.test(get(t, 'verified_status.value', ''))
    }
  })
  return testScores
}

export const parseSeminarySummaries = (seminary, questions) => {
  const convertFromQuestion = (q) => {
    if (!q) return undefined
    const findById = id => q.elements.find(e => e.id === id)
    const comments = findById('seminary_comments')
    const graduate = findById('graduate_flag')
    const email = findById('teacher_email')
    const name = findById('teacher_name')
    const yearsTeaching = findById('teacher_years_teaching')
    const responseCriteria = [
      'No basis / Not applicable',
      'Below Average',
      'Average',
      'Above Average <br/>(top 30%)',
      'Excellent <br/>(top 10%)',
      'Outstanding <br/>(top 5%)',
      'One of the top few I\'ve encountered <br/>(top 1%)'
    ]
    const questions = q.elements.filter(e => /^question_/.test(e.id)).map(
      q => {
        return {
          prompt: q.prompt,
          response: responseCriteria.findIndex(c => c === q.response)
        }
      }
    )
    return {
      comments,
      graduate,
      instructor: {
        name,
        email,
        yearsTeaching
      },
      questions,
      responseCriteria
    }
  }
  const parseTermInfo = i => {
    return {
      sessions: get(i, 'sessions.value'),
      reading: get(i, 'reading.value'),
      term: get(i, 'term.value'),
      recognition: get(i, 'recognition.value'),
      attendance: get(i, 'attendance.value'),
      grade: get(i, 'grade.value'),
      absences: get(i, 'absences.value'),
      credit: get(i, 'credit.value')
    }
  }
  const parseSeminaryClass = c => {
    return {
      seminaryClass: get(c, 'seminary_class.value'),
      termInfo: get(c, 'term_info', []).map(parseTermInfo),
      academicYear: get(c, 'academic_year.value')
    }
  }
  const recommendation = convertFromQuestion(questions.SEMINARY)
  return {
    seminaryType: get(seminary, 'seminary_type.description'),
    seminaryClasses: get(seminary, 'seminary_classes', []).map(parseSeminaryClass),
    yearsCompleted: get(seminary, 'years_completed.value'),
    graduationStatus: get(seminary, 'graduation_status.value'),
    teacherEmail: get(seminary, 'teacher_email.value'),
    teacherName: get(seminary, 'teacher_name.value'),
    seminaryExplanation: get(seminary, 'seminary_explanation.value'),
    attendancePercentage: get(seminary, 'attendance_percentage.value'),
    dateTimeReported: get(seminary, 'date_time_reported.value'),
    recommendation
  }
}

export const parseInstituteSummaries = institute => {
  const parseClass = c => {
    return {
      className: get(c, 'institute_class.value'),
      academicYear: get(c, 'academic_year.value'),
      grade: get(c, 'grade.value'),
      forCredit: get(c, 'for_credit.value')
    }
  }
  return {
    attendance: get(institute, 'institute_attendance.value'),
    explanation: get(institute, 'institute_explanation.value'),
    classes: get(institute, 'institute_classes', []).map(parseClass)
  }
}

export const parseGovernmentRecords = g => {
  const citizenship = {
    citizenshipStatus: get(g, 'citizenship_status.value'),
    citizenshipStatusDescription: get(g, 'citizenship_status.description'),
    visaTypeSource: get(g, 'visa_type_source.value'),
    visaType: get(g, 'visa_type.value'),
    birthCity: get(g, 'birth_city.value'),
    citizenshipCountryCode: get(g, 'citizenship_country_code.value'),
    citizenshipCountry: get(g, 'citizenship_country_code.description')
  }
  return citizenship
}

export const parsePersonalRecords = p => {
  const dateOfBirth = get(p, 'date_of_birth.value')
  const age = dateOfBirth ? ageFn(dateOfBirth) : ''
  return {
    sex: get(p, 'sex.value'),
    dateOfBirth,
    age,
    religion: get(p, 'religion_code.description'),
    religionTenure: get(p, 'religion_tenure.value')
  }
}

export const parseQuestions = q => {
  const parseElement = el => ({
    id: get(el, 'id.value'),
    type: get(el, 'type.value'),
    prompt: get(el, 'prompt.value'),
    explanation: get(el, 'explanation.value'),
    response: get(el, 'response.value'),
    responseSet: get(el, 'response_set'),
    ordinal: get(el, 'ordinal.value'),
    limit: get(el, 'limit.value'),
    responseCriteria: get(el, 'response_criteria', []).map(({ value, label }) => ({ value, label })),
    responseRange: get(el, 'response_range.value')
  })

  const parseGroup = el => ({
    id: get(q, 'id.value'),
    heading: get(q, 'heading.value'),
    description: get(q, 'description.value'),
    name: get(q, 'name.value'),
    ordinal: get(q, 'ordinal.value'),
    elements: get(el, 'elements', []).map(parseElement)
  })

  const questions = get(q, 'values', []).reduce((o, q) => {
    const name = get(q, 'name.value')
    if (!name) {
      return o
    }
    return Object.assign(o, {
      [name]: {
        name,
        description: get(q, 'description.value'),
        type: get(q, 'type.value'),
        uuid: get(q, 'uuid.value'),
        elements: get(q, 'elements', []).map(parseElement),
        groups: get(q, 'groups', []).map(parseGroup)
      }
    })
  }, {})
  return questions
}

export const questionResponses = ({ elements = [] }) => elements.reduce((s, el) => s.concat(el.response), '')
export const wc = str => str.length > 0 ? str.split(/\s+/).length : 0

export const parsePathway = g => {
  return {
    credits: get(g, 'pw_credits.value'),
    gpa: get(g, 'pw_gpa.value'),
    completion: get(g, 'pw_completion.value'),
    site: get(g, 'pw_site.value')
  }
}
