import get from 'lodash.get'
import { dateTimeFormat } from '../dateTimeFormat'

const parsePacketNote = packet => {
  const createdBy = get(packet, 'id_created_by.description', '')
  const dateTimeCreated = dateTimeFormat(get(packet, 'date_time_created.value', new Date()))
  const note = get(packet, 'action.value', '')
  const qualifier = get(packet, 'qualifier.value', '')
  const reference = get(packet, 'reference.value', '')
  const type = get(packet, 'type.value', '')
  const visibility = get(packet, 'visibility.value', '')
  return {
    createdBy,
    dateTimeCreated,
    note,
    qualifier,
    reference,
    type,
    visibility
  }
}

export const parsePacketNotes = (packet = {}) => {
  const values = get(packet, 'actions.values', [])
  return values.map(parsePacketNote)
}

const parseApplicationNote = action => {
  const rawNote = get(action, 'action_message.value', '')
  const note = rawNote.charAt(0) === '{'
    ? JSON.parse(rawNote)
    : rawNote
  const dateTimeCreated = dateTimeFormat(get(action, 'action_date_time.value', new Date()))
  const createdBy = get(action, 'action_by_id.description')
  const reference = get(action, 'deliverable_id.value')
  const type = get(action, 'action.value')
  return {
    note,
    dateTimeCreated,
    createdBy,
    reference,
    type
  }
}

export const parseApplicationNotes = (application = {}) => {
  const values = get(application, 'actions.values', [])
  return values.map(parseApplicationNote)
}
