import { parsePacketNotes } from './notes'
import parseDecisionHistory from './decisionHistory'
import get from 'lodash.get'

export const parseFlag = f => {
  const dateTimeCleared = get(f, 'date_time_cleared.value')
  const setBy = get(f, 'id_set_by.description')
  const setById = get(f, 'id_set_by.value')
  const name = get(f, 'name.value')
  const value = get(f, 'value.value')
  const dateTimeSet = get(f, 'date_time_set.value')
  const clearedBy = get(f, 'id_cleared_by.description')
  const clearedById = get(f, 'id_cleared_by.value')
  const type = get(f, 'type.value')
  const state = get(f, 'state.value')
  const message = get(f, 'message.value')
  return {
    dateTimeCleared,
    setBy,
    setById,
    name,
    value,
    dateTimeSet,
    clearedBy,
    clearedById,
    type,
    state,
    message
  }
}

export const parseFlags = (f = {}) => {
  const values = get(f, 'values', [])
  return values.map(parseFlag)
}

export const parseDecision = (d = {}) => {
  const dateTimeUpdated = get(d, 'date_time_updated.value')
  const explanation = get(d, 'explanation.value')
  const model = get(d, 'model.value')
  const reason = get(d, 'reason.value')
  const updatedBy = get(d, 'id_updated_by.description')
  const updatedById = get(d, 'id_updated_by.value')
  const value = get(d, 'value.value')

  return {
    dateTimeUpdated,
    explanation,
    model,
    reason,
    updatedBy,
    updatedById,
    value
  }
}

export const parseProcess = (p) => {
  return {
    dateTimeCleared: get(p, 'date_time_cleared.value'),
    dateTimeSet: get(p, 'date_time_set.value'),
    name: get(p, 'name.value'),
    setBy: get(p, 'id_set_by.description'),
    setById: get(p, 'id_set_by.value'),
    status: get(p, 'status.value')
  }
}

export const parseProcesses = (processes = {}) => {
  const values = get(processes, 'values', [])
  return values.map(parseProcess)
}

export const parsePacket = (packet) => {
  if (!packet) return { flags: [], notes: [] }
  const notes = parsePacketNotes(get(packet, 'packet'))
  const flags = parseFlags(get(packet, 'flags', {}))
  const decisionHistory = parseDecisionHistory(get(packet, 'history', {}))
  return {
    institution: get(packet, 'basic.institution.value'),
    admitPeriod: get(packet, 'basic.admit_period.value'),
    applicantId: get(packet, 'basic.applicant_id.value'),
    applicantMajor: get(packet, 'basic.applicant_major.value'),
    applicantName: get(packet, 'basic.applicant_name.value'),
    applicantType: get(packet, 'basic.applicant_type.value'),
    hasFlags: packet.flags.values.length > 0,
    flags,
    decision: parseDecision(packet.decision),
    decisionHistory,
    notes,
    processes: parseProcesses(packet.processes)
  }
}
