import get from 'lodash.get'

const parsePacket = packet => {
  const basic = get(packet, 'basic', {})
  const flags = get(packet, 'basic.flags', [])
  const assignedTo = flags.filter(f => /^[0-9]{9}$/.test(f))[0]
  const population = get(basic, 'applicant_population.value', '')
  return {
    admit_period: get(basic, 'admit_period.value'),
    applicant_id: get(basic, 'applicant_id.value'),
    applicant_major: get(basic, 'applicant_major.value'),
    applicant_name: get(basic, 'applicant_name.value'),
    applicant_type: get(basic, 'applicant_type.value'),
    application_uuid: get(basic, 'application_uuid.value'),
    applicant_sex: get(basic, 'applicant_sex.value'),
    applicant_birth_date: get(basic, 'applicant_birth_date.value'),
    applicant_net_id: get(basic, 'net_id.value'),
    applicant_gpa: get(basic, 'applicant_gpa.value'),
    cumulative_college_gpa: get(basic, 'cumulative_college_gpa.value'),
    applicant_act: get(basic, 'applicant_act.value'),
    assigned_to: assignedTo,
    created_date_time: get(basic, 'created_date_time.value'),
    institution: get(basic, 'institution.value'),
    is_hawaii: population === 'HI',
    is_international: population === 'IA',
    next_action_date: get(basic, 'next_action.value'),
    set_flags: flags
  }
}

const parseValues = json => {
  const values = get(json, 'values', [])
  return values.map(parsePacket)
}

const parseTotal = json => {
  const metadata = get(json, 'metadata', [])
  return get(metadata, 'collection_size', 0)
}

const parseStats = json => {
  const stats = get(json, 'stats', [])
  const admitPeriods = stats.map(s => s.key).filter(s => /^[0-9]{5}$/.test(s))
  return stats.reduce((acc, curr) => {
    const { key, value } = curr
    return Object.assign(acc, { [key]: Math.floor(value) })
  }, { admitPeriods })
}

export default (json) => {
  const data = parseValues(json)
  const total = parseTotal(json)
  const stats = parseStats(json)
  return { data, total, stats }
}
