import * as activity from './activity'
import * as background from './background'
import * as bio from './bio'
import * as citizenship from './citizenship'
import * as collegeSummaries from './collegeSummaries'
import * as essay from './essay'
import * as highSchoolSummaries from './highSchoolSummaries'
import * as institute from './institute'
import * as mission from './mission'
import * as personalRecords from './personalRecords'
import * as questions from './questions'
import * as seminary from './seminary'
import * as testScores from './testScores'

// initial state
export const state = {
  addressLines: [],
  admitPeriod: '',
  applicantId: '',
  applicantMajor: '',
  applicantName: '',
  applicantType: '',
  applicationStatus: '',
  city: '',
  emailAddress: '',
  decision: '',
  decisionQualifier: '',
  decisionDateTime: '',
  netId: '',
  personId: '',
  phoneNumber: '',
  whatsappNumber: '',
  preferredName: '',
  previousName: '',
  schoolPreferenceOrder: [],
  sortName: '',
  state: '',
  submittedOn: ''
}

// getters
export const getters = {
  collegeSummaries (state) {
    return state.collegeSummaries.colleges
  },
  highSchoolSummaries (state) {
    return state.highSchoolSummaries.highSchools
  },
  testScores (state) {
    return state.testScores.scores
  }
}

// mutations
export const mutations = {
  setApplication (state, application) {
    state.addressLines = application.addressLines
    state.admitPeriod = application.admitPeriod
    state.applicantId = application.applicantId
    state.applicantMajor = application.applicantMajor
    state.applicantName = application.applicantName
    state.applicantType = application.applicantType
    state.applicationStatus = application.applicationStatus
    state.city = application.city
    state.emailAddress = application.emailAddress
    state.decision = application.decision
    state.decisionQualifier = application.decisionQualifier
    state.decisionDateTime = application.decisionDateTime
    state.netId = application.netId
    state.personId = application.personId
    state.phoneNumber = application.phoneNumber
    state.whatsappNumber = application.whatsappNumber
    state.preferredName = application.preferredName
    state.previousName = application.previousName
    state.schoolPreferenceOrder = application.schoolPreferenceOrder
    state.sortName = application.sortName
    state.state = application.state
    state.submittedOn = application.submittedOn
  },
  resetApplication (state) {
    state.addressLines = []
    state.admitPeriod = ''
    state.applicantId = ''
    state.applicantMajor = ''
    state.applicantName = ''
    state.applicantType = ''
    state.applicationStatus = ''
    state.city = ''
    state.emailAddress = ''
    state.decision = ''
    state.decisionQualifier = ''
    state.decisionDateTime = ''
    state.netId = ''
    state.personId = ''
    state.phoneNumber = ''
    state.whatsappNumber = ''
    state.preferredName = ''
    state.previousName = ''
    state.schoolPreferenceOrder = []
    state.sortName = ''
    state.state = ''
    state.submittedOn = ''
  }
}

// actions
export const actions = {
  resetApplication ({ commit }) {
    commit('resetApplication')
    commit('resetActivity')
    commit('resetBackground')
    commit('resetBio')
    commit('resetCitizenship')
    commit('resetCollegeSummaries')
    commit('resetEssay')
    commit('resetHighSchoolSummaries')
    commit('resetInstitute')
    commit('resetMission')
    commit('resetPersonalRecords')
    commit('resetQuestions')
    commit('resetSeminary')
    commit('resetTestScores')
  },
  loadApplication ({ commit }, application) {
    commit('setApplication', application.basic)
    commit('setActivity', application.activity)
    commit('setBackground', application.background)
    commit('setBio', application.bio)
    commit('setCitizenship', application.citizenship)
    commit('setCollegeCumulative', application.collegeCumulative)
    commit('setCollegeSummaries', application.collegeSummaries)
    commit('setEssay', application.essay)
    commit('setHighSchoolSummaries', application.highSchoolSummaries)
    commit('setInstitute', application.institute)
    commit('setMission', application.mission)
    commit('setPersonalRecords', application.personalRecords)
    commit('setQuestions', application.questions)
    commit('setSeminary', application.seminary)
    commit('setTestScores', application.testScores)
  }
}

export const modules = {
  activity,
  background,
  bio,
  citizenship,
  collegeSummaries,
  essay,
  highSchoolSummaries,
  institute,
  mission,
  personalRecords,
  questions,
  seminary,
  testScores
}
