import Vue from 'vue'
// initial state
// data shape: {
//   applicant_name,
//   application_uuid,
//   applicant_id,
//   applicant_type,
//   applicant_major,
//   applicant_net_id,
//   admit_period,
//   institution,
//   next_action_date,
//   is_international
// }
// query shape: {
//  listType,
//  limit,
//  offset,
//  sort,
//  order,
//  filters: [{type, value}...]
// }
export const state = {
  data: [],
  prevPage: [],
  nextPage: [],
  total: 0,
  fetchedOn: undefined,
  query: {
    listType: '',
    limit: 25,
    offset: 0,
    sort: 'applicant_name',
    order: 'asc',
    filters: []
  }
}

export const getters = {
  packetListQuery: state => state.query,
  prevApplication: state => currentId => {
    const idx = state.data.findIndex(a => a.application_uuid === currentId)
    if (idx <= 0) return null
    return state.data[idx - 1].application_uuid
  },
  nextApplication: state => currentId => {
    const idx = state.data.findIndex(a => a.application_uuid === currentId)
    if (idx < 0 || idx + 1 >= state.data.length) return null
    return state.data[idx + 1].application_uuid
  },
  packetListFilters (state) {
    return state.query.filters
  }
}

export const mutations = {
  resetPacketList (state) {
    state.data = []
    state.prevPage = []
    state.nextPage = []
    state.total = 0
    state.fetchedOn = undefined
    state.query = {
      listType: '',
      limit: 25,
      offset: 0,
      sort: 'applicant_name',
      order: 'asc',
      filters: []
    }
  },
  setPacketListData (state, data = []) {
    state.data = data
  },
  setPacketListTotal (state, total) {
    state.total = total
  },
  setPacketListCacheControl (state, { fetchedOn, query }) {
    state.fetchedOn = fetchedOn
    const {
      listType,
      limit,
      offset,
      sort,
      order
    } = query

    state.query.listType = listType
    state.query.limit = Number.parseInt(limit)
    state.query.offset = Number.parseInt(offset)
    state.query.sort = sort
    state.query.order = order
  },
  modifyPacketListQuery (state, query) {
    const {
      listType,
      limit,
      offset,
      sort,
      order
    } = query

    state.query.listType = listType
    state.query.limit = limit
    state.query.offset = offset
    state.query.sort = sort
    state.query.order = order
  },
  clearPacketListFilter (state, { type, all = false }) {
    if (all) {
      state.query.filters = []
      return
    }
    const currIndex = state.query.filters.findIndex(f => f.type === type)
    if (currIndex >= 0) {
      state.query.filters.splice(currIndex, 1)
    }
  },
  setPacketListFilter (state, { type, value }) {
    const currIndex = state.query.filters.findIndex(f => f.type === type)
    if (currIndex >= 0) {
      Vue.set(state.query.filters, currIndex, { type, value })
    } else {
      state.query.filters.push({ type, value })
    }
  }
}
