const KEY = 'edu.byu.dp.hawaii.ui'

export const state = {
  viewedApplications: [],
  admitPeriod: '',
  institution: ''
}

export const mutations = {
  resetUi (state) {
    state.viewedApplications = []
    state.admitPeriod = ''
    state.institution = ''
  },
  setUi (state, ui) {
    if (Array.isArray(ui.viewedApplications)) {
      state.viewedApplications = ui.viewedApplications.slice(0)
    }
    if (ui.admitPeriod !== undefined) {
      state.admitPeriod = ui.admitPeriod
    }
    if (ui.institution !== undefined) {
      state.institution = ui.institution
    }
  },
  viewApplication (state, { appId, name }) {
    if (state.viewedApplications.find(i => i.appId === appId)) {
      return
    }
    state.viewedApplications.push({ appId, name })
    if (state.viewedApplications.length > 5) {
      // only retain 5 most recently viewed apps
      state.viewedApplications = state.viewedApplications.slice(-5)
    }
  },
  selectAdmitPeriod (state, admitPeriod) {
    state.admitPeriod = admitPeriod
  },
  setInstitution (state, institution) {
    state.institution = institution
  }
}

export const actions = {
  preserveUiSettings ({ state }) {
    const storage = window.localStorage
    storage.setItem(KEY, JSON.stringify(state))
  },
  loadUiSettings ({ commit }) {
    const storage = window.localStorage
    const value = storage.getItem(KEY)
    if (value) {
      commit('setUi', JSON.parse(value))
    }
  }
}
